package main;

import meerkat.crypto.mixnet.Mix2ZeroKnowledgeVerifier;
import meerkat.protobuf.Crypto;
import meerkat.protobuf.Mixing;
import necessary.AsyncBulletinBoardClient;
import necessary.BulletinBoardClient;
import necessary.CompleteBatch;
import qilin.util.Pair;
import verifier.VerifyTable;

import java.util.Arrays;
import java.util.List;

/**
 * Created by Tzlil on 12/17/2015.
 */
public class BatchHandler implements AsyncBulletinBoardClient.ClientCallback<CompleteBatch> {

    private Pair<Mixing.ZeroKnowledgeProof[][], Crypto.RerandomizableEncryptedMessage[][]> mixerOutput;
    private boolean msgReceived;
    private Throwable t;
    private CompleteBatch msg;

    private final int n, layers;
    private final Mix2ZeroKnowledgeVerifier verifier;

    public BatchHandler(int n, int layers, Mix2ZeroKnowledgeVerifier verifier) {
        this.mixerOutput = null;
        this.n = n;
        this.layers = layers;
        this.msgReceived = false;
        this.t = null;
        this.verifier = verifier;
    }

    @Override
    public void handleCallback(CompleteBatch msg) {
        this.msg = msg;
        this.msgReceived = true;
    }

    @Override
    public void handleFailure(Throwable t) {
        this.t = t;
        this.msgReceived = true;
    }


    public boolean isMsgReceived() {
        return msgReceived;
    }

    private void convertMessage() throws Exception {
        BatchConverter batchConverter = new BatchConverter(n,layers);
        mixerOutput = batchConverter.batchDataList2MixerOutput(msg.getBatchDataList());
    }

    public boolean verifyTable() throws Exception {
        if (mixerOutput == null) {
            convertMessage();
        }
        return VerifyTable.verifyTable(verifier, n, mixerOutput);
    }

    public List<Crypto.RerandomizableEncryptedMessage> getInputForMixer() throws Throwable {
        if (t != null) {
            throw t;
        }
        if (mixerOutput == null) {
            convertMessage();
            if (!VerifyTable.verifyTable(verifier, n, mixerOutput)) {
                throw new Exception("in valid table");
            }
        }
        return Arrays.asList(mixerOutput.b[mixerOutput.b.length - 1]);
    }
}

