package mixer;

import com.google.protobuf.ByteString;
import com.google.protobuf.InvalidProtocolBufferException;
import meerkat.crypto.concrete.ECElGamalEncryption;
import meerkat.crypto.mixnet.Mix2ZeroKnowledgeProver;
import meerkat.crypto.mixnet.Mix2ZeroKnowledgeVerifier;
import meerkat.protobuf.ConcreteCrypto;
import meerkat.protobuf.Crypto;
import meerkat.protobuf.Voting;
import org.bouncycastle.math.ec.ECPoint;
import org.junit.Before;
import org.junit.Test;
import prover.Prover;
import qilin.primitives.RandomOracle;
import qilin.primitives.concrete.DigestOracle;
import qilin.primitives.concrete.ECElGamal;
import qilin.primitives.concrete.ECGroup;
import verifier.Verifier;

import java.math.BigInteger;
import java.util.Random;

/**
 * Created by Tzlil on 1/17/2016.
 */
public class RerandomizeTest {

    Random rand;
    ECElGamal.SK key;
    ECGroup group;
    ECElGamalEncryption enc;
    ConcreteCrypto.ElGamalPublicKey serializedPk;
    ECPoint g ;
    ECPoint h ;
    @Before
    public void setup() throws Exception {
        rand = new Random();
        group = new ECGroup("secp256k1");
        BigInteger sk = ECElGamal.generateSecretKey(group, rand);
        key = new ECElGamal.SK(group, sk);
        serializedPk = Utiles.serializePk(group, key);
        enc = new ECElGamalEncryption();
        enc.init(serializedPk);
        RandomOracle randomOracle = new DigestOracle();
        g = group.getGenerator();
        h = enc.getElGamalPK().getPK();
    }

    private ECPoint convert2ECPoint(ByteString bs){
        return group.decode(bs.toByteArray());
    }
    
    public void oneRerandomizeTest() throws InvalidProtocolBufferException {
        Voting.PlaintextBallot msg = Utiles.genRandomBallot(2,3,16); // 2 questions with 3 answers each, in range 0-15.

        Crypto.EncryptionRandomness r = enc.generateRandomness(rand);


        Crypto.RerandomizableEncryptedMessage e = enc.encrypt(msg, enc.generateRandomness(rand));
        Crypto.RerandomizableEncryptedMessage eNew = enc.rerandomize(e, r);

        assert (Utiles.decrypt(Voting.PlaintextBallot.class, key, group, e).equals(msg));
        assert (Utiles.decrypt(Voting.PlaintextBallot.class, key, group, eNew).equals(msg));

        ConcreteCrypto.ElGamalCiphertext eElGamal = ECElGamalEncryption.rerandomizableEncryptedMessage2ElGamalCiphertext(e);
        ConcreteCrypto.ElGamalCiphertext eNewElGamal = ECElGamalEncryption.rerandomizableEncryptedMessage2ElGamalCiphertext(eNew);

        assert (g.multiply(new BigInteger(r.getData().toByteArray())).equals(
                group.add(convert2ECPoint(eNewElGamal.getC1()),group.negate(convert2ECPoint(eElGamal.getC1())))));
        assert (h.multiply(new BigInteger(r.getData().toByteArray())).equals(
                group.add(convert2ECPoint(eNewElGamal.getC2()), group.negate(convert2ECPoint(eElGamal.getC2())))));
    }

    @Test
    public void rerandomizeTest() throws InvalidProtocolBufferException {

        int tests = 1000;

        for (int i = 0; i < tests; i ++){
            System.out.println("rerandomiz test #" + i);
            oneRerandomizeTest();
        }
    }
}
